package pay.platform.mapper;

import com.mybatisflex.core.BaseMapper;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Nullable;
import lombok.NonNull;
import org.apache.ibatis.annotations.Mapper;
import pay.platform.domain.SysUser;
import pay.platform.domain.table.SysRoleTableDef;
import pay.platform.domain.table.SysUserRoleTableDef;
import pay.platform.domain.table.SysUserTableDef;

import java.util.HashSet;
import java.util.Set;

/**
 * 系统 - 用户表 映射层。
 *
 * @author admin
 * @since 1.0
 */
@Mapper
public interface SysUserMapper extends BaseMapper<SysUser> {

    /**
     * 根据用户名获取已授权的角色代码
     *
     * @param username 用户名
     * @return 已授权的角色代码
     */
    @Nullable
    default Set<String> getRoleCodeByUsername(@NonNull String username) {
        QueryWrapper query = QueryWrapper.create().select(SysRoleTableDef.SYS_ROLE.ROLE_CODE).from(SysRoleTableDef.SYS_ROLE, SysUserRoleTableDef.SYS_USER_ROLE, SysUserTableDef.SYS_USER).where(SysRoleTableDef.SYS_ROLE.ID.eq(SysUserRoleTableDef.SYS_USER_ROLE.ROLE_ID)).and(SysRoleTableDef.SYS_ROLE.ENABLE.eq(1)).and(SysUserRoleTableDef.SYS_USER_ROLE.USER_ID.eq(SysUserTableDef.SYS_USER.ID)).and(SysUserTableDef.SYS_USER.USERNAME.eq(username));
        return new HashSet<>(selectListByQueryAs(query, String.class));
    }

    /**
     * 根据用户ID 获取对应的角色ID集合
     *
     * @param userId 用户ID
     * @return 角色ID集合
     */
    @Nullable
    default Set<String> queryUserRole(@NonNull String userId) {
        QueryWrapper query = QueryWrapper.create()
                .select(SysUserRoleTableDef.SYS_USER_ROLE.ROLE_ID)
                .from(SysUserRoleTableDef.SYS_USER_ROLE, SysRoleTableDef.SYS_ROLE)
                .where(SysUserRoleTableDef.SYS_USER_ROLE.ROLE_ID.eq(SysRoleTableDef.SYS_ROLE.ID))
                .and(SysRoleTableDef.SYS_ROLE.ENABLE.eq(1))
                .and(SysUserRoleTableDef.SYS_USER_ROLE.USER_ID.eq(userId));
        return new HashSet<>(selectListByQueryAs(query, String.class));
    }

    default void resetPassword(String username, String newPassword) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysUserTableDef.SYS_USER).where(SysUserTableDef.SYS_USER.USERNAME.eq(username));
        SysUser sysUser = selectOneByQuery(queryWrapper);
        sysUser.setPassword(newPassword);
        update(sysUser);
    }
}
