package pay.platform.api.system.cache;

import cn.bzvs.json.JsonUtils;
import com.alibaba.fastjson2.TypeReference;
import pay.platform.api.system.model.vo.VueMenuRouteVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Optional;

/**
 * 缓存当前用户路由信息
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class RouteCache implements CacheUtil {
    private final PlatformCache cache;

    /**
     * 缓存用户路由
     *
     * @param userid 用户路由
     * @param routes 路由
     */
    public void putCache(String userid, List<VueMenuRouteVO> routes) {
        log.info("【用户路由】缓存用户: {} 路由信息, 缓存时间: 2小时", userid);
        String cacheKey = getCacheKey(RouteCacheKey.default_key, userid);
        String json = JsonUtils.toJsonString(routes);
        cache.set(cacheKey, json, 60 * 60 * 2);
    }

    /**
     * 获取用户路由
     *
     * @param userid 用户ID
     * @return 用户路由
     */
    public Optional<List<VueMenuRouteVO>> getCache(String userid) {
        log.info("【用户路由】获取用户: {} 缓存的路由信息", userid);
        String cacheKey = getCacheKey(RouteCacheKey.default_key, userid);
        Optional<String> jsonOptional = cache.getStr(cacheKey);
        return jsonOptional.flatMap(e -> Optional.ofNullable(JsonUtils.stringToBean(e, new TypeReference<>() {
        })));
    }

    /**
     * 清理用户路由
     *
     * @param userid .
     */
    public void removeCache(String userid) {
        log.info("【用户路由】清理用户: {} 缓存的路由信息", userid);
        String cacheKey = getCacheKey(RouteCacheKey.default_key, userid);
        cache.del(cacheKey);
    }


    public enum RouteCacheKey implements KeyValue {
        default_key("route", "当前用户路由");
        private final String prefix;
        private final String desc;

        RouteCacheKey(String prefix, String desc) {
            this.prefix = prefix;
            this.desc = desc;
        }

        @Override
        public String getPrefix() {
            return prefix;
        }

        @Override
        public String getDesc() {
            return desc;
        }
    }
}
