package pay.platform.api.system.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import pay.platform.api.system.model.vo.SelectOptionVO;
import pay.platform.core.common.Result;
import pay.platform.core.enums.ConfigTypeEnums;
import pay.platform.core.enums.EnabledEnums;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 系统：常量
 */
@RestController
@RequestMapping("/sys/enums")
@RequiredArgsConstructor
@Tag(name = "系统：常量")
public class SysEnumsController {


    /**
     * 状态 options
     *
     * @return .
     */
    @GetMapping("/enabled")
    @Operation(summary = "状态列表")
    public Result<List<SelectOptionVO>> enabledOptions() {
        List<SelectOptionVO> res = Stream.of(EnabledEnums.values()).map(v -> SelectOptionVO.builder().value(STR."\{v.getValue()}").name(v.getName()).build()).collect(Collectors.toList());
        return Result.OK(res);
    }

    /**
     * 配置类型
     *
     * @return .
     */
    @Operation(summary = "配置类型")
    @GetMapping("/config_type")
    public Result<List<SelectOptionVO>> configTypeOptions() {
        List<SelectOptionVO> res = Stream.of(ConfigTypeEnums.values()).map(v -> SelectOptionVO.builder().name(v.getName()).value(STR."\{v.getValue()}").build()).collect(Collectors.toList());
        return Result.OK(res);
    }
}
