package pay.platform.api.system.servcie;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.util.StringUtil;
import pay.platform.api.system.cache.ConfigCache;
import pay.platform.api.system.model.query.ConfigQuery;
import pay.platform.api.system.model.vo.ConfigVO;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.common.domain.BaseService;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.domain.SysConfig;
import pay.platform.domain.table.SysConfigTableDef;
import pay.platform.mapper.SysConfigMapper;
import jakarta.annotation.Nonnull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

/**
 * 系统 - 配置表 服务层实现。
 *
 * @author ADMIN
 * @since 1.0
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class SysConfigService extends BaseService<SysConfigMapper, SysConfig> {

    private final ConfigCache cache;

    /**
     * 分页查询
     *
     * @param query .
     * @return .
     */
    public Result<BasePage<ConfigVO>> queryPage(ConfigQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysConfigTableDef.SYS_CONFIG).where("1 = 1").and(SysConfigTableDef.SYS_CONFIG.TYPE.eq(query.getType(), query.getType() != null)).and(SysConfigTableDef.SYS_CONFIG.CODE.like(query.getCode(), StringUtil.isNotBlank(query.getCode())));
        Page<SysConfig> page = mapper.paginate(new Page<>(query.getCurrent(), query.getSize()), queryWrapper);
        List<ConfigVO> vos = BeanUtil.copyToList(page.getRecords(), ConfigVO.class);
        BasePage<ConfigVO> _page = new BasePage<>(page.getPageNumber(), page.getPageSize(), page.getTotalRow(), vos);
        return Result.OK(_page);
    }

    /**
     * 列表查询
     *
     * @param query .
     * @return .
     */
    public Result<List<ConfigVO>> queryList(ConfigQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysConfigTableDef.SYS_CONFIG).where("1 = 1").and(SysConfigTableDef.SYS_CONFIG.TYPE.eq(query.getType(), query.getType() != null)).and(SysConfigTableDef.SYS_CONFIG.CODE.like(query.getCode(), StringUtil.isNotBlank(query.getCode())));
        List<SysConfig> list = mapper.selectListByQuery(queryWrapper);
        List<ConfigVO> vos = BeanUtil.copyToList(list, ConfigVO.class);
        return Result.OK(vos);
    }

    /**
     * 删除
     *
     * @param ids .
     * @return .
     */
    public boolean removeByIds(List<String> ids) {
        if (null == ids || ids.isEmpty()) {
            return false;
        }
        List<SysConfig> configs = listByIds(ids);
        if (CollectionUtil.isNotEmpty(configs)) {
            configs.stream().map(SysConfig::getCode).forEach(cache::delCache);
        }
        return super.removeByIds(ids);
    }

    /**
     * 刷新缓存
     *
     * @param id 缓存ID
     * @return .
     */
    public Result<String> refreshCache(@Nonnull String id) {
        SysConfig config = getById(id);
        if (null == config) {
            return Result.NG("配置不存在");
        }
        cache.putCache(config);
        return Result.OK("刷新成功");
    }

    /**
     * 获取缓存
     *
     * @param code 编号
     * @return .
     */
    public Result<String> getByCode(@Nonnull String code) {
        Optional<String> optional = cache.getCache(code);
        if (optional.isPresent()) {
            return Result.OK(optional.get());
        }
        SysConfig sysConfig = mapper.selectOneByCondition(SysConfigTableDef.SYS_CONFIG.CODE.eq(code));
        if (sysConfig == null) {
            return Result.NG("配置不存在");
        }
        return Result.OK("成功", sysConfig.getValue());
    }

    /**
     * 获取缓存值
     *
     * @param code 编码
     * @return .
     */
    public String getValue(@Nonnull String code) {
        Optional<String> optional = cache.getCache(code);
        if (optional.isPresent()) {
            return optional.get();
        }
        SysConfig sysConfig = mapper.selectOneByCondition(SysConfigTableDef.SYS_CONFIG.CODE.eq(code));
        if (sysConfig == null) {
            return null;
        }
        return sysConfig.getValue();
    }

    /**
     * 更新缓存值
     *
     * @param code  编码
     * @param value 值
     * @return .
     */
    public boolean updateValue(String code, String value) {
        return updateValue(code, value, false);
    }

    /**
     * 更新缓存值
     *
     * @param code  编码
     * @param value 值
     * @param init  初始化
     * @return .
     */
    public boolean updateValue(@Nonnull String code, @Nonnull String value, boolean init) {
        SysConfig sysConfig = mapper.selectOneByCondition(SysConfigTableDef.SYS_CONFIG.CODE.eq(code));
        if (sysConfig == null && init) {
            sysConfig = new SysConfig();
            sysConfig.setType(1);
            sysConfig.setCode(code);
            sysConfig.setValue(value);
            sysConfig.setCreateTime(LocalDateTime.now());
            sysConfig.setCreateUser(SecurityUtil.getCurrentUsername());
            sysConfig.setUpdateTime(LocalDateTime.now());
            sysConfig.setUpdateUser(SecurityUtil.getCurrentUsername());
            return save(sysConfig);
        }
        sysConfig.setValue(value);
        sysConfig.setUpdateTime(LocalDateTime.now());
        sysConfig.setUpdateUser(SecurityUtil.getCurrentUsername());
        return updateById(sysConfig);
    }
}